package com.qhtx.eta.common.exception;

import com.qhtx.eta.common.eunms.ErrorEnum;

/**
 * ETA 业务异常类
 */
public class ETAException extends RuntimeException {

    private static final long serialVersionUID = 1L;

    /**
     * 错误代码，用于标识具体的业务错误类型。
     */
    private String errorCode;

    /**
     * 错误消息，描述业务异常的具体情况。
     */
    private String errorMessage;

    /**
     * 可选的附加信息，如受影响的实体ID、操作详情等。
     */
    private String additionalInfo;

    public ETAException(ErrorEnum errorEnum) {
        super(errorEnum.getMsg());
        this.errorCode = errorEnum.getCode();
        this.errorMessage = errorEnum.getMsg();
    }

    public ETAException(ErrorEnum errorEnum, String additionalInfo) {
        super(errorEnum.getMsg());
        this.errorCode = errorEnum.getCode();
        this.errorMessage = errorEnum.getMsg();
        this.additionalInfo = additionalInfo;
    }

    public ETAException(String errorCode, String errorMessage) {
        super(errorMessage);
        this.errorCode = errorCode;
        this.errorMessage = errorMessage;
    }

    public ETAException(String errorCode, String errorMessage, String additionalInfo) {
        super(errorMessage);
        this.errorCode = errorCode;
        this.errorMessage = errorMessage;
        this.additionalInfo = additionalInfo;
    }

    public ETAException(Throwable cause) {
        super(cause);
        this.errorCode = ErrorEnum.SYSTEM_ERROR.getCode();
        this.errorMessage = cause.getMessage();
    }

    public ETAException(String errorCode, String errorMessage, Throwable cause) {
        super(errorMessage, cause);
        this.errorCode = errorCode;
        this.errorMessage = errorMessage;
    }

    // Add other constructors as needed, depending on your requirements.

    public String getErrorCode() {
        return errorCode;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public Object getAdditionalInfo() {
        return additionalInfo;
    }

    @Override
    public String getMessage() {
        return String.format("Business Error: [Code=%s] %s", errorCode, errorMessage);
    }

    @Override
    public String toString() {
        return String.format("BusinessException{errorCode='%s', errorMessage='%s', additionalInfo=%s}", errorCode, errorMessage, additionalInfo);
    }
}
