package com.qhtx.eta.common.utils;

import com.qhtx.eta.common.entity.SignatureParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

@Slf4j
public class SignatureUtil {

    private static final String HMAC_SHA256_ALGORITHM = "HmacSHA256";

    /**
     * 生成HMAC-SHA256签名。
     *
     * @return 签名字符串（Base64编码）
     * @throws Exception 如果签名过程中发生错误
     */
    public static String generateSignature(SignatureParam signatureParam) {
        try {
            return sign(signatureParam.toString(), signatureParam.getSecretKey());
        } catch (Exception e) {
            log.warn("生成签名失败:{}", e.getMessage());
        }
        return "";
    }

    public static String generateNonce() {
        return RandomStringUtils.randomAlphabetic(32);
    }

    public static Long generateTimestamp() {
        return System.currentTimeMillis() / 1000;
    }


    public static String sign(String signStr, String apiKey) throws NoSuchAlgorithmException, InvalidKeyException {
        // 使用HMAC-SHA256算法计算签名
        Mac mac = Mac.getInstance(HMAC_SHA256_ALGORITHM);
        SecretKeySpec secretKey = new SecretKeySpec(apiKey.getBytes(StandardCharsets.UTF_8), HMAC_SHA256_ALGORITHM);
        mac.init(secretKey);
        byte[] sign = mac.doFinal(signStr.getBytes(StandardCharsets.UTF_8));

        return StringUtils.replaceEach(Base64.getEncoder().encodeToString(sign), new String[]{"+", "/"}, new String[]{"-", "_"});
    }
}
