package com.qhtx.eta.domain.api.http.response.eta;

import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.domain.entity.QuotaDataDTO;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Data
@Slf4j
public class QuotaDataResponse extends ETAHttpResponse<ETAData, QuotaDataDTO> {

    private static final long serialVersionUID = -1579830772947683015L;

    public List<QuotaDataDTO> convertDataToDTO() {
        return this.getData().stream().map(item -> {
            QuotaDataDTO result = new QuotaDataDTO();
            result.setDataTime(StrToDate(item.getDataTime()));
            result.setDataValue(new BigDecimal(item.getValue()));
            result.setUpdateTime(StrToTimestamp(item.getUpdateTime()));
            return result;
        }).filter(item -> item.getDataTime() != null).collect(Collectors.toList());
    }

    private Date StrToDate(String dateTime) {
        try {
            if (StringUtils.isEmpty(dateTime)) {
                return null;
            }
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern(ETAConstants.DATE_PATTERN);
            return Date.from(LocalDate.parse(dateTime, formatter).atStartOfDay(ZoneId.of(ETAConstants.TIME_ZONE)).toInstant());
        } catch (Exception e) {
            log.warn("字段日期格式错误:{}", dateTime);
            return null;
        }
    }

    private Date StrToTimestamp(String dateTime) {
        try {
            if (StringUtils.isEmpty(dateTime)) {
                return null;
            }
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern(ETAConstants.TIMESTAMP_PATTERN);
            return Date.from(LocalDateTime.parse(dateTime, formatter).atZone(ZoneId.of(ETAConstants.TIME_ZONE)).toInstant());
        } catch (Exception e) {
            return null;
        }
    }
}
