package com.qhtx.eta.domain.service.impl;

import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.utils.MapUtils;
import com.qhtx.eta.domain.convert.EtaApiQuotaInfoConverter;
import com.qhtx.eta.domain.convert.LinkDataDTOConverter;
import com.qhtx.eta.domain.convert.TDampDwLinkInfoConverter;
import com.qhtx.eta.domain.entity.LinkDataDTO;
import com.qhtx.eta.domain.entity.LinkFieldDTO;
import com.qhtx.eta.domain.entity.LinkInfoDTO;
import com.qhtx.eta.domain.entity.QuotaInfoDTO;
import com.qhtx.eta.domain.utils.RedisUtils;
import com.qhtx.eta.infra.entity.dw.TDampDwIndexLink;
import com.qhtx.eta.infra.entity.dw.TDampDwLinkInfo;
import com.qhtx.eta.infra.service.TDampDwIndexLinkService;
import com.qhtx.eta.infra.service.TDampDwLinkDataService;
import com.qhtx.eta.infra.service.TDampDwLinkInfoService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

@Service
@Slf4j
@Data
public class TDampLinkService {

    @Resource
    private TDampDwLinkInfoService tdampDwLinkInfoService;

    @Resource
    private TDampDwLinkDataService tdampDwLinkDataService;

    @Resource
    private TDampDwIndexLinkService tDampDwIndexLinkService;
    // ETA 维表和对接码的映射关系
    private static Map<Integer, LinkFieldDTO> linkFieldMap = new ConcurrentHashMap<>();

    @Resource
    private RedisUtils redisUtils;
    @Resource(name = "IndexSyncThreadPool")
    private ThreadPoolExecutor indexSyncThreadPool;

    /**
     * 初始化ETA 维表和对接码的映射关系
     */
    public void initLinkInfo() {
        TDampDwLinkInfo linkInfo = tdampDwLinkInfoService.queryByLinkPrefix(ETAConstants.ETA_LINK_PREFIX);
        if (linkInfo == null) {
            log.warn("未配置ETA对接信息");
            return;
        }
        LinkInfoDTO linkInfoDTO = TDampDwLinkInfoConverter.INSTANCE.convertToDTO(linkInfo);
        for (LinkFieldDTO field : linkInfoDTO.getLinkFields()) {
            linkFieldMap.put(field.getFiledType(), field);
        }
        //设置已经同步的指标
        List<TDampDwIndexLink> indexLinks = tDampDwIndexLinkService.queryAll();
        indexLinks.forEach(indexLink -> {
            int uniqueId = Integer.parseInt(indexLink.getLinkCode().substring(indexLink.getLinkCode().indexOf("ETA") + 3));
            redisUtils.setBit(redisUtils.indexSyncKey(), uniqueId, true);
            log.info("对接码：{}对应指标数据已同步,不在进行同步处理,指标编码{}", indexLink.getLinkCode(), indexLink.getIndexCode());
        });
    }

    public List<LinkDataDTO> getLinkDataList() {
        long total = tdampDwLinkDataService.count();
        if (total > ETAConstants.INDEX_SYNC_LIMIT) {
            List<CompletableFuture<List<LinkDataDTO>>> futures = new ArrayList<>();
            long chunk = (total + ETAConstants.INDEX_SYNC_LIMIT - 1) / ETAConstants.INDEX_SYNC_LIMIT;
            for (int i = 0; i < chunk; i++) {
                int current = i;
                CompletableFuture<List<LinkDataDTO>> future = CompletableFuture.supplyAsync(() -> {
                    try {
                        int offset = current * ETAConstants.INDEX_SYNC_LIMIT;
                        return LinkDataDTOConverter.INSTANCE.convertEntityToDTOList(tdampDwLinkDataService.queryByLimit(offset, ETAConstants.INDEX_SYNC_LIMIT));
                    } catch (Exception e) {
                        log.error("获取对接码数据列表失败", e);
                        return Collections.emptyList();
                    }
                }, indexSyncThreadPool);
                futures.add(future);
            }
            List<LinkDataDTO> list = futures.stream().map(CompletableFuture::join).flatMap(List::stream).collect(Collectors.toList());
            return list.stream().filter(linkDataDTO -> !redisUtils.getBit(redisUtils.indexSyncKey(), Integer.parseInt(linkDataDTO.getLinkCode().substring(linkDataDTO.getLinkCode().indexOf("ETA") + 3)))).collect(Collectors.toList());
        } else {
            List<LinkDataDTO> list = LinkDataDTOConverter.INSTANCE.convertEntityToDTOList(tdampDwLinkDataService.queryAll());
            return list.stream().filter(linkDataDTO -> !redisUtils.getBit(redisUtils.indexSyncKey(), Integer.parseInt(linkDataDTO.getLinkCode().substring(linkDataDTO.getLinkCode().indexOf("ETA") + 3)))).collect(Collectors.toList());
        }
    }

    public LinkFieldDTO getLinkField(int filedType) {
        return linkFieldMap.get(filedType);
    }
}
