package com.qhtx.eta.infra.aspect;

import com.qhtx.eta.infra.annotation.UseDataSource;
import com.qhtx.eta.infra.datasource.DataSourceContextHolder;
import com.qhtx.eta.common.eunms.DataSourceType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.util.Optional;

@Component
@Aspect
public class DataSourceSwitchAspect {

    @Around("execution(* com.qhtx.eta.infra.service.*.*(..))")
    //@Around("@annotation(com.qhtx.eta.infra.annotation.UseDataSource)")
    public Object switchDataSource(ProceedingJoinPoint joinPoint) throws Throwable {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        UseDataSource dataSource = Optional.ofNullable(signature.getMethod().getAnnotation(UseDataSource.class)).orElseGet(() -> signature.getMethod().getDeclaringClass().getAnnotation(UseDataSource.class));
        DataSourceType dataSourceType = dataSource != null ? dataSource.dataSourceType() : DataSourceType.ODS;
        if (dataSourceType == null) {
            throw new ETAException(ErrorEnum.UNKNOWN_DATASOURCE_ERROR);
        }
        // 切换数据源
        DataSourceContextHolder.setDataSourceType(dataSourceType);

        try {
            // 执行目标方法
            Object obj = joinPoint.proceed();
            return obj;
        } finally {
            // 清理ThreadLocal中的数据源信息
            DataSourceContextHolder.clearDataSourceType();
        }
    }
}
