package com.qhtx.eta.infra.service.impl;

import com.qhtx.eta.common.annotation.DataOperator;
import com.qhtx.eta.common.eunms.DataSourceType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.infra.annotation.UseDataSource;
import com.qhtx.eta.infra.entity.ods.EtaApiQuotaInfo;
import com.qhtx.eta.common.eunms.ETAApiType;
import com.qhtx.eta.infra.mapper.EtaApiQuotaInfoDao;
import com.qhtx.eta.infra.service.EtaApiQuotaInfoService;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.List;

/**
 * (EtaApiQuotaInfo)表服务实现类
 *
 * @author makejava
 * @since 2024-04-29 13:52:38
 */
@Service("etaApiQuotaInfoService")
@UseDataSource(dataSourceType = DataSourceType.ODS)
@Slf4j
public class EtaApiQuotaInfoServiceImpl implements EtaApiQuotaInfoService {
    @Resource
    private TransactionTemplate transactionTemplate;
    @Resource
    private EtaApiQuotaInfoDao etaApiQuotaInfoDao;

    /**
     * 通过ID查询单条数据
     *
     * @param quoteId 主键
     * @return 实例对象
     */
    @Override
    public EtaApiQuotaInfo queryById(Integer quoteId) {
        return this.etaApiQuotaInfoDao.queryById(quoteId);
    }

    @Override
    public long countQuota() {
        return etaApiQuotaInfoDao.count();
    }


    /**
     * 新增数据
     *
     * @param etaApiQuotaInfo 实例对象
     * @return 实例对象
     */
    @Override
    public EtaApiQuotaInfo insert(EtaApiQuotaInfo etaApiQuotaInfo) {
        this.etaApiQuotaInfoDao.insert(etaApiQuotaInfo);
        return etaApiQuotaInfo;
    }

    /**
     * 修改数据
     *
     * @param etaApiQuotaInfo 实例对象
     * @return 实例对象
     */
    @Override
    public EtaApiQuotaInfo update(EtaApiQuotaInfo etaApiQuotaInfo) {
        this.etaApiQuotaInfoDao.update(etaApiQuotaInfo);
        return this.queryById(etaApiQuotaInfo.getQuotaId());
    }

    /**
     * 通过主键删除数据
     *
     * @param quoteId 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Integer quoteId) {
        return this.etaApiQuotaInfoDao.deleteById(quoteId) > 0;
    }

    @Override
    @DataOperator(apiType = ETAApiType.QUOTA, fields = {"uniqueCode"}, dataType = EtaApiQuotaInfo.class)
    public void batchInsertOrUpdate(List<EtaApiQuotaInfo> quotaInfoDTOS) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            public void doInTransactionWithoutResult(@NotNull TransactionStatus transactionStatus) {
                try {
                    etaApiQuotaInfoDao.insertOrUpdateBatch(quotaInfoDTOS);
                } catch (Exception e) {
                    log.warn(String.format(ErrorEnum.DATASOURCE_EXECUTE_ERROR.getMsg(), e.getMessage()));
                    throw e;
                }

            }
        });
    }

    @Override
    public List<EtaApiQuotaInfo> queryQuotaList(EtaApiQuotaInfo etaApiQuotaInfo) {
        return etaApiQuotaInfoDao.queryQuotaList(etaApiQuotaInfo);
    }

    @Override
    public List<EtaApiQuotaInfo> queryQuotaListPage(int offset, int limit) {
        return etaApiQuotaInfoDao.queryQuotaListPage(offset, limit);
    }

    @Override
    public List<EtaApiQuotaInfo> getAllQuotaList() {
        return etaApiQuotaInfoDao.getAllQuotaList();
    }
}
