package com.qhtx.eta.infra.service.impl;


import com.qhtx.eta.common.eunms.DataSourceType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.infra.annotation.UseDataSource;
import com.qhtx.eta.infra.entity.dw.TEtaIndexPushTask;
import com.qhtx.eta.infra.enums.Frequency;
import com.qhtx.eta.infra.mapper.TEtaIndexPushTaskDao;
import com.qhtx.eta.infra.service.TEtaIndexPushTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * (TEtaIndexPushTask)表服务实现类
 *
 * @author makejava
 * @since 2024-09-30 13:43:27
 */
@Service("tEtaIndexPushTaskService")
@UseDataSource(dataSourceType = DataSourceType.DW)
@Slf4j
public class TEtaIndexPushTaskServiceImpl implements TEtaIndexPushTaskService {
    @Resource
    private TEtaIndexPushTaskDao tEtaIndexPushTaskDao;

    /**
     * 新增数据
     *
     * @param tEtaIndexPushTask 实例对象
     * @return 实例对象
     */
    @Override
    public TEtaIndexPushTask addTask(TEtaIndexPushTask tEtaIndexPushTask) {
        TEtaIndexPushTask dbTask = tEtaIndexPushTaskDao.getByIndexCode(tEtaIndexPushTask.getIndexCode());
        if (dbTask != null) {
            log.info("指标{}已存在,不新增", tEtaIndexPushTask.getIndexCode());
            throw new ETAException(ErrorEnum.INDEX_AllREADY_ADD);
        }
        tEtaIndexPushTaskDao.insert(tEtaIndexPushTask);
        return tEtaIndexPushTask;
    }

    /**
     * 修改数据
     *
     * @param tEtaIndexPushTask 实例对象
     * @return 实例对象
     */
    @Override
    public void update(TEtaIndexPushTask tEtaIndexPushTask) {
        this.tEtaIndexPushTaskDao.update(tEtaIndexPushTask);
    }

    @Override
    public List<TEtaIndexPushTask> getAllTask() {
        return tEtaIndexPushTaskDao.getAllTask().stream().filter(item -> {
            String frequencyStr = String.format("%s度", item.getFrequency());
            Frequency frequency = Frequency.getFrequencyByDesc(frequencyStr);
            if (frequency == null) {
                log.warn("指标{} 频度获取失败,不支持的频度类型{},不同步数据", item.getIndexCode(), item.getFrequency());
                return false;
            }
            Date lastPushTime = item.getLastPushTime();
            return needPush(lastPushTime);
        }).collect(Collectors.toList());
    }

    @Override
    public TEtaIndexPushTask getByIndexCode(String indexCode) {
        return tEtaIndexPushTaskDao.getByIndexCode(indexCode);
    }

    private boolean needPush(Date lastPushTime) {
        if (lastPushTime == null) {
            return true;
        }
        int numberOfPeriods = -1;
        LocalDate today = LocalDate.now();
        LocalDate localDate = lastPushTime.toInstant().atZone(ZoneId.of("Asia/Shanghai")).toLocalDate();
        return localDate.minusDays(numberOfPeriods).compareTo(today) <= 0;
    }

    private boolean needPushByFrequency(Date lastPushTime, Frequency frequency) {
        if (lastPushTime == null) {
            return true;
        }
        int numberOfPeriods = -1;
        if (frequency == null) {
            frequency = Frequency.DAILY;
        }
        LocalDate today = LocalDate.now();
        LocalDate localDate = lastPushTime.toInstant().atZone(ZoneId.of("Asia/Shanghai")).toLocalDate();
        switch (frequency) {
            case DAILY:
                return localDate.minusDays(numberOfPeriods).compareTo(today) <= 0;
            case WEEKLY:
                return localDate.minusWeeks(numberOfPeriods).compareTo(today) <= 0;
            case MONTHLY:
                return localDate.minusMonths(numberOfPeriods).compareTo(today) <= 0;
            case YEARLY:
                return localDate.minusYears(numberOfPeriods).compareTo(today) <= 0;
            case SEMI_ANNUAL:
                return localDate.minusMonths(6L * numberOfPeriods).compareTo(today) <= 0;
            case QUARTERLY:
                return localDate.minusMonths(3L * numberOfPeriods).compareTo(today) <= 0;
            case DECADELY:
                return localDate.minusDays(10L * numberOfPeriods).compareTo(today) <= 0;
            default:
                throw new IllegalArgumentException("Unsupported frequency: " + frequency);
        }
    }
}
